﻿-- Broker_ProfessionsMenu
-- Author: Yargoro@Arathor-EU

local _,ADDON = ...
local MODNAME = "BrokerNameToggle"
local core   = LibStub("AceAddon-3.0"):NewAddon(MODNAME,"AceEvent-3.0") --, "AceTimer-3.0")
local L      = LibStub("AceLocale-3.0"):GetLocale(MODNAME)
local QTip   = LibStub:GetLibrary("LibQTip-1.0")
local LDB    = LibStub:GetLibrary("LibDataBroker-1.1")

local radioActivecl = "|cffff0000"
local radioInactivecl = "|cffaaff00"---|cffffff00"

ADDON.version = "Version: 7.0.3.2 (2016.Jul.20)"
ADDON.debug = false

function ADDON:Debug(msg)
	if self.debug and DEFAULT_CHAT_FRAME then
		DEFAULT_CHAT_FRAME:AddMessage(MODNAME .. " (dbg): " .. tostring(msg), 1.0, 0.37, 0.37)
	end
end 

--------------------------------------------------------------------------------------------------------
-- Variables and defaults
--------------------------------------------------------------------------------------------------------

local IN_COMBAT = 0
local OUT_OF_COMBAT = 1
local state = OUT_OF_COMBAT

local toggleCVars =
{
	["Players"] =
	{
		"UnitNameOwn",
		"UnitNameEnemyPlayerName",
		"UnitNameEnemyPetName",
		"UnitNameEnemyGuardianName",
		"UnitNameEnemyMinionName",
		"UnitNameEnemyTotemName",
--		"UnitNameForceHideMinus",
		"UnitNameFriendlyPlayerName",
		"UnitNameFriendlyPetName",
		"UnitNameFriendlyGuardianName",
		"UnitNameFriendlyMinionName",
		"UnitNameFriendlyTotemName",

		"UnitNameNonCombatCreatureName",
		"UnitNamePlayerGuild",
		"UnitNameGuildTitle",
		"UnitNamePlayerPVPTitle",
	},
	["Toggles"] =
	{
		["NPC Names"] = 
		{
			["Quest NPCs"] =
			{
				["UnitNameFriendlySpecialNPCName"] = true,
				["UnitNameNPC"] = false,
				["UnitNameHostleNPC"] = false,
				["UnitNameInteractiveNPC"] = false,
				["ShowQuestUnitCircles"] = false,
			},  
			["Hostile and Quest NPCs"] =
			{
				["UnitNameFriendlySpecialNPCName"] = true,
				["UnitNameHostleNPC"] = true,
				["UnitNameInteractiveNPC"] = false,
				["UnitNameNPC"] = false,
				["ShowQuestUnitCircles"] = true,
			},
			["Hostile,Quest,and Interactive NPCs"] =
			{
				["UnitNameFriendlySpecialNPCName"] = true,
				["UnitNameHostleNPC"] = true,
				["UnitNameInteractiveNPC"] = true,
				["UnitNameNPC"] = false,
				["ShowQuestUnitCircles"] = true,
			},
			["All NPCs"] =
			{
				["UnitNameFriendlySpecialNPCName"] = false,
				["UnitNameHostleNPC"] = false,
				["UnitNameInteractiveNPC"] = false,
				["UnitNameNPC"] = true,
				["ShowQuestUnitCircles"] = true,
			},	
			["None"] =
			{
				["UnitNameFriendlySpecialNPCName"] = false,				 
				["UnitNameHostleNPC"] = false,
				["UnitNameInteractiveNPC"] = false,
				["UnitNameNPC"] = false,
				["ShowQuestUnitCircles"] = true,
			}
		}
	},
	["Nameplates"] =
	{
		"nameplateShowFriends",
		"nameplateShowFriendlyPets",
		"nameplateShowFriendlyGuardians",
		"nameplateShowFriendlyTotems",
		"nameplateShowEnemies",
		"nameplateShowEnemyPets",
		"nameplateShowEnemyGuardians",
		"nameplateShowEnemyTotems",
		"nameplateShowEnemyMinus"
	},
}

local radioCVars =
{
}

ADDON.obj = LDB:NewDataObject(MODNAME,
{
	type = "data source",
	icon = "Interface\\Icons\\Ability_Hunter_Pathfinding",
	label = "NameToggle",
	text = "NameToggle",
	OnEnter = function(frame)
			ADDON:CreateTooltip()
			ADDON.tooltip:Clear()
			ADDON.tooltip:SmartAnchorTo(frame)
			ADDON.tooltip:SetAutoHideDelay(0.1, frame)
			ADDON:ShowTooltip()
		end,
	OnClick = function(frame,button)
		if (ADDON.tooltip == nil) then
			ADDON:CreateTooltip()
		end
		if button=="LeftButton" then
--			if IsShiftKeyDown() then
--				ADDON:Toggle("UnitNameNPC")
--			else
			if IsControlKeyDown() then
				ADDON:Toggle("UnitNameOwn")
			elseif IsAltKeyDown() then
				ADDON:Toggle("UnitNameEnemyPlayerName")
			else
				ADDON:Toggle("UnitNameFriendlyPlayerName")
			end
		end
		ADDON:ShowTooltip()
		end,
	}
)

function ADDON:OnInitialize()
	self:RegisterEvent("PLAYER_REGEN_ENABLED","OnRegenEnable")
	self:RegisterEvent("PLAYER_REGEN_DISABLED","OnRegenDisable")
end

function ADDON:OnRegenEnable()
	state = OUT_OF_COMBAT
end

function ADDON:OnRegenDisable()
	state = IN_COMBAT
end

local function ToggleTooltip(var)
	ADDON:Toggle(var._OnMouseDown_arg.Var)
	ADDON.tooltip:SetCell(var._OnMouseDown_arg.line,2,GetCVarBool(var._OnMouseDown_arg.Var) and L["shown"] or L["hidden"])
	ADDON.tooltip:SetScrollStep(15)
end

local function ToggleRadioTip(var)
	local radiooptionsubcatvalue = var._OnMouseDown_arg.p1
	local radiooptionsubcat = var._OnMouseDown_arg.p2
	local radiosubcatvalue = var._OnMouseDown_arg.p3
	local radiosubcatvalueline = var._OnMouseDown_arg.p4
	ADDON:ToggleRadio(radiooptionsubcatvalue) 
	ADDON:RefreshRadioCategory(radiosubcatvalue,radiosubcatvalueline)
	ADDON.tooltip:SetScrollStep(15)
end

function ADDON:Toggle(var)
	if (state == OUT_OF_COMBAT) then
		SetCVar(var, GetCVar(var) == "1" and 0 or 1)
	end
end

function ADDON:ToggleRadio(radioTable)
	if (state == OUT_OF_COMBAT) then
		for radiocat,radiocatvalue in pairs(radioTable) do
			SetCVar(radiocat,radiocatvalue)
		end
	end
end

function ADDON:RefreshRadioCategory(radioCategoryTable,startline)
	if (state == OUT_OF_COMBAT)	then
		local line=startline+1
		for	radiooptionsubcat,radiooptionsubcatvalue in pairs(radioCategoryTable) do
			ADDON.tooltip:SetCell(line,1,"   "..(IsRadioActive(radiooptionsubcatvalue) and radioActivecl or radioInactivecl)..radiooptionsubcat.."|r")
			line=line+1
		end
	end
end

function ADDON:CreateTooltip()
	local tooltip = QTip:Acquire(MODNAME.."Tooltip", 2, "LEFT", "RIGHT")
	ADDON.tooltip = tooltip
end

function ADDON:ShowTooltip()
	ADDON.tooltip:Clear()
	ADDON.tooltip:AddHeader(MODNAME)
	
	for cat,catvalue in pairs(toggleCVars) do
		ADDON.tooltip:AddSeparator()
		if (cat == "Toggles") then
			for radiosubcat,radiosubcatvalue in pairs(catvalue) do
				local y = ADDON.tooltip:AddLine("|cffffff00"..L[radiosubcat].."|r")
				for	radiooptionsubcat,radiooptionsubcatvalue in pairs(radiosubcatvalue) do
					local y2 = ADDON.tooltip:AddLine("   "..(IsRadioActive(radiooptionsubcatvalue) and radioActivecl or radioInactivecl)..L[radiooptionsubcat].."|r")
					ADDON.tooltip:SetLineScript(y2,"OnMouseDown",ToggleRadioTip,{p1=radiooptionsubcatvalue,p2=radiooptionsubcat,p3=radiosubcatvalue,p4=y,line=y2})
				end
			end
		else
			ADDON.tooltip:AddLine("|cffffff00"..cat.."|r")
			for subcat,subcatvalue in pairs(catvalue) do
				local y = ADDON.tooltip:AddLine(L[subcatvalue],GetCVarBool(subcatvalue) and L["shown"] or L["hidden"])
				ADDON.tooltip:SetLineScript(y,"OnMouseDown",ToggleTooltip,{Var=subcatvalue,line=y})
			end
		end
	end

	ADDON.tooltip:AddSeparator()
	ADDON.tooltip:AddLine()
	ADDON.tooltip:AddLine("|cff00ff00"..L["hint"].."|r")
	ADDON.tooltip:AddLine()
	ADDON.tooltip:Show()
end

function IsRadioActiveText(radioTable)
	return IsRadioActive(radioTable) and "true" or "false"
end

function IsRadioActive(radioTable)
	local result = true
	for radiocat,radiocatvalue in pairs(radioTable) do
		if not GetCVarBool(radiocat) == radiocatvalue then result = false end
	end
--	ADDON:Debug("IsRadioActive("..tostring(radioTable).."):"..tostring(result));
	return result
end
